<?php
namespace Service;

use \Service\View;
use \Service\Widget;
use \Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Service\Indicator;
use \Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Environment;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Dashboard {

	/** @var \Service\View $viewService */
	private $viewService;
	/** @var \Service\Widget $widgetService */
	private $widgetService;
	/** @var \Service\ViewType $viewTypeService */
	private $viewTypeService;
	/** @var \Service\ImportHistory */
    protected $importHistoryService;
    /** @var \Util\Environment $environment */
	protected $environment;
	/** @var \Service\Hierarchy $hierarchyService */
    protected $hierarchyService;

	protected $defaultOrgManagers;

	private $indicatorService;

	private $dataSourceName = "view";

	private $organizationManager;

	const TWO_MINUTES = 120;
    const VIEW_TYPE_GENERATED_MENU = 'G';

	public function __construct(View $viewService, Widget $widgetService, ViewType $viewTypeService,
									array $defaultOrgManagers, Indicator $indicatorService, Manager $organizationManager,
									ImportHistory $importHistoryService, Environment $environment, Hierarchy $hierarchyService) {
		$this->viewService = $viewService;
		$this->widgetService = $widgetService;
		$this->viewTypeService = $viewTypeService;
		$this->defaultOrgManagers = $defaultOrgManagers;
		$this->indicatorService = $indicatorService;
		$this->organizationManager = $organizationManager;
		$this->importHistoryService = $importHistoryService;
		$this->environment = $environment;
		$this->hierarchyService = $hierarchyService;
	}

	public function buildDashboardsByMenu($menuId, $filterParams) {
		$menu = $this->getGeneratedMenu($menuId);
        $menuType = $menu['type'];
		$dashboards = array(array(),array());
		$errorMessage = "";
		foreach($menu['dashboards'] as $dashboard){
			$dash=$this->buildDashboards($this->viewService->findDashboardViewsByMenu($dashboard, $menuType), $filterParams);
			if(empty($dash[0]['error'])){
				array_push($dashboards[0],$dash[0]);
			} else {
				$errorMessage .= $dash[0]['name'].", ";
			}
		}
		if(!empty($errorMessage)){
			$dashboards[1] = array('error' => 'DASHBOARDS_WITHOUT_ACCESS', 'd' => $errorMessage);
		}
		return $dashboards;
	}

	public function getGeneratedMenu($menuId){
		$filterCriteria = new FilterCriteria('view');
		$filterCriteria->addCondition("_id", $menuId);
		$menu = $this->viewService->findBy($filterCriteria);
		return reset($menu);
	}

	public function buildMainDashboards() {
		set_time_limit(Dashboard::TWO_MINUTES);
		$mainDashboardViews = $this->viewService->getMainDashboardViews();
		$dashboard = $this->buildDashboards($mainDashboardViews, null);
		return $dashboard;
	}

    public function buildGenericDashboards() {
		$mainDashboardViews = $this->viewService->getGenericDashboardViews();
		$dashboard = $this->buildDashboards($mainDashboardViews, null);
		return $dashboard;
	}
	
	public function buildPreviewDashboards($id) {
		$this->organizationManager->createDatabase(Utilities::getImportDefaultDatabase());
		$mainDashboardViews = $this->viewService->getDashboardViews(null, false, $id);
		$dashboard = $this->buildDashboards($mainDashboardViews, null);
		return $dashboard;
	}

	public function fixBugGaugeInAllIndicators(&$indicator){
		$this->fixBugGauge($indicator);
        if(isset($indicator['chartSeries']) && is_array($indicator['chartSeries'])){
            foreach($indicator['chartSeries'] as &$chart){
                $this->fixBugGauge($chart);
            }
        } 
	}

	public function fixBugGauge(&$chart){
		if(isset($chart['serieRanges'])){
			$chart['zmartSerieRanges'] = $chart['serieRanges'];
			foreach($chart['zmartSerieRanges'] as $i => $range){
				if(!is_numeric($range)){
					$chart['serieRanges'][$i] = "0";
				}
			}
		}
		if(isset($chart["serieMaxValue"]) && !is_numeric($chart["serieMaxValue"])){
			$chart["zmartSerieMaxValue"] = $chart["serieMaxValue"];
			$chart["serieMaxValue"] = 1;
		}
		if(isset($chart["serieMinValue"]) && !is_numeric($chart["serieMinValue"])){
			$chart["zmartSerieMinValue"] = $chart["serieMinValue"];
			$chart["serieMinValue"] = 0;
		}
	}

	protected function buildDashboards($dashboards, $filterParams) {
		$buildedDashboards = array();
		$lastImportDates = [];

		if (!empty($dashboards)) {
			foreach ($dashboards as $dashboard) {
				$buildedDashboards[] = $this->buildDashboard($dashboard, $filterParams, $lastImportDates);
			}
		}

		return $buildedDashboards;
	}

	protected function buildDashboard($dashboardObj, $filterParams, $lastImportDates = array()) {
		if(!empty($dashboardObj["error"]) && $dashboardObj["error"]){
			$dashboard = $dashboardObj;
		} else {
			$dashboard = array(
				'_id' => $dashboardObj['_id'],
				'name' => $dashboardObj['name'],
				'indicators' => array(),
				'order' => isset($dashboardObj['order']) ? $dashboardObj['order'] : null,
				'template' => isset($dashboardObj['dashboardTemplate']) ? $dashboardObj['dashboardTemplate'] : 'tab'
			);

			if($dashboard['template'] === 'accordion'){
				$dashboard['accordionHeaderProperties'] = array(
					'color' => isset($dashboardObj['headerColor']) ? $dashboardObj['headerColor'] : null,
					'gradient' => isset($dashboardObj['headerGradient']) ? $dashboardObj['headerGradient'] : null,
					'icon' => isset($dashboardObj['headerBackgroundIcon']) ? $dashboardObj['headerBackgroundIcon'] : null,
					'image' => isset($dashboardObj['headerBackgroundImg']) ? $dashboardObj['headerBackgroundImg'] : null,
					'dataSource' => isset($dashboardObj['dataSource']) ? $dashboardObj['dataSource'] : null
				);

				$this->addDashboardIndicators($dashboardObj, $dashboard, $filterParams);
			}

			if(isset($dashboardObj['indicators'])) {
				unset($dashboardObj['indicators']);
			}
		}
		return $dashboard;
	}

	protected function addDashboardIndicators($dashboardObj, &$dashboard, $filterParams) {
		if(isset($dashboardObj['indicators'])) {
			foreach ($dashboardObj['indicators'] as $indicator) {
				$indicator = $this->setIndicatorFilterParameters($indicator, $filterParams);
				$indicator = $this->buildDashboardIndicator($indicator);
				$dataSourceId = $indicator['dataSourceId'];
				$indicator['lastImportation'] = isset($lastImportDates[$dataSourceId]) ? $lastImportDates[$dataSourceId] : null;
				$dashboard["indicators"][] = $indicator;
				
			}
		}
	}

	public function buildWidgetIndicator($dashboard, $userStructureHierarchy = null, $isViewPreview = false){
		$error = [];
		foreach ($dashboard as $dashboardObj) {
			$widgetIndicator = [];
			$hierarchyList = [];
			if(isset($dashboardObj['indicators'])) {
				foreach ($dashboardObj['indicators'] as $indicator) {
					if(empty($hierarchyList[$indicator["indicator"]])){
						$hierarchyList[$indicator["indicator"]] = $this->hierarchyService->getIndicatorHierarchyByLoggedUser($indicator["indicator"], null);
					}
					$lastImportDates = isset($indicator['dataSource']) ? $this->importHistoryService->getLastImportDatesByDataSource($indicator['dataSource']) : [];
					$indicator = $this->setIndicatorFilterParameters($indicator, null);
					try {
						$indicator = $this->buildDashboardIndicator($indicator, $userStructureHierarchy, $isViewPreview, $hierarchyList);
						$dataSourceId = $indicator['dataSourceId'];
						$indicator['lastImportation'] = isset($lastImportDates[$dataSourceId]) ? $lastImportDates[$dataSourceId] : null;
						$dashboard["indicators"][] = $indicator;
					} catch(\Exception $e) {
						if($e->getCode() == 1 && isset($indicator["indicator_label"]) && isset($indicator["widget_label"])){
							$error[] = array(
								"message"=>"INDICATOR_VIEW_NOT_FOUND",
								"params" => array("v"=>$indicator["indicator_label"], "i" => $indicator["widget_label"])
							);
						}else{
							$error[] = $e->getMessage();
						}
					}
				array_push($widgetIndicator, $indicator);
				}
				$dashboardObj['indicators'] = $widgetIndicator;
			}
		}
		return array($dashboard,$error);
	}

	public function checkDrillDownIndicators($dashboardsIds){
		$drillDownIndicators=[];
		$indicatorIds=[];
		foreach($dashboardsIds as $dashboardId){
			$filterCriteria = new FilterCriteria($this->dataSourceName, array(array("columnName"=>"_id", "value" => $dashboardId, "operator" => "=")));
			$dashboard = $this->getDataFromMultipleOrgs($filterCriteria)[0];
			if(!empty($dashboard["indicators"])){
				foreach($dashboard["indicators"] as $indicator){
					if(!in_array($indicator["indicator"],$indicatorIds)){
						array_push($indicatorIds,$indicator["indicator"]);
					}
				}
			}
		}
		$drillDownIndicators = $this->indicatorService->checkDrillDownIndicators($indicatorIds);
		$drillDownIndicatorsNames = array();
		if(!empty($drillDownIndicators)){
			$filterCriteria = new FilterCriteria("indicator", array(array("columnName"=>"_id", "value" => $drillDownIndicators, "operator" => "IN")));
			$indicators = $this->getDataFromMultipleOrgs($filterCriteria);
			foreach($indicators as $indicator){
				if(in_array($indicator["_id"],$drillDownIndicators)){
					array_push($drillDownIndicatorsNames,array("_id" => $indicator["_id"], "name" => $indicator["name"]));
				}	
			}
		}
		return array($drillDownIndicators,$drillDownIndicatorsNames);
	}

	public function saveDefaultOrgMenuOrgView($menu){
		return $this->organizationManager->persist(new DataSet('view', [$menu]));
	}

	public function setLogImport($dashboardImported,$indicatorImported,$dataSourceImported){
        foreach ($dashboardImported as $dashboard) {
            $this->viewService->setLog(null,$dashboard['_id']);
	    }

        foreach ($indicatorImported as $indicator) {
            $this->indicatorService->setLog(null,$indicator['_id']);
            $widgets = $this->widgetService->findWidgetsByIndicator($indicator['_id']);
            foreach ($widgets as $widget) {
                $this->widgetService->setLog(null,$widget['_id']);
            }
        }
        $dataSourceService = InstanceManager::getInstance()->getService('\Service\DataSource');
        foreach ($dataSourceImported as $dataSource) {
            $dataSourceService->setLog(null,$dataSource['_id']);
        }
    }

	public function saveDefaultOrgDashboardsAndIndicators($dashboardsId, $additionalIndicators = null){
		$dashboardsToImport = array();

		foreach($dashboardsId as $dashboardId){
			$filterCriteria = new FilterCriteria($this->dataSourceName, array(array("columnName"=>"_id", "value" => $dashboardId, "operator" => "=")));
			array_push($dashboardsToImport, $this->getDataFromMultipleOrgs($filterCriteria)[0]);
		}

		$dashboardsToImport = $this->removeObjectsAlreadyImported($dashboardsToImport);
		list($dashboardsToImport,$viewIds,$indicatorImported,$dataSourceImported) = $this->importDashboardIndicators($dashboardsToImport);

		foreach($dashboardsToImport as &$dashboard){
			$dashboard['oldId'] = $dashboard["_id"];
			unset($dashboard["_id"]);
			unset($dashboard["menu"]);
		}

		if(!empty($additionalIndicators) && is_array($additionalIndicators)){
			foreach($additionalIndicators as $additionalIndicator){
				$indicatorAlreadyOnDb = $this->organizationManager->findBy(new FilterCriteria("indicator",
					array(array("columnName"=>"oldId", "value" => $additionalIndicator, "operator" => "="))
				))->getRows();
				if(empty($indicatorAlreadyOnDb)){
					$indicator =$this->indicatorService->saveDefaultOrgIndicatorsAndDataSources([$additionalIndicator]);
					foreach($indicator["widget"] as $widget){
						$viewIds[] = $widget;
					}
                	$indicatorImported[] = $indicator["indicator"][0];
                	if(!empty($indicator["newDatasource"])){
                	    $dataSourceImported[] = $indicator["newDatasource"][0];
                	}
				}
			}
		}

		$uniqueViews=[];
		$uniqueViewsList=[];
		foreach($viewIds as $view){
			if(!in_array($view["_id"],$uniqueViewsList)){
				array_push($uniqueViewsList,$view["_id"]);
				array_push($uniqueViews,$view);
			}
		}
		$this->indicatorService->setAllDrillDowns(array("widget"=>$uniqueViews));
		$persistedDashboards = $this->organizationManager->persist(new DataSet('view', $dashboardsToImport));
		return array($persistedDashboards,$indicatorImported,$dataSourceImported);
	}

	protected function importDashboardIndicators($dashboardsToImport){
		$viewIds=[];
        $indicatorImported=[];
        $dataSourceImported=[];
		foreach($dashboardsToImport as $id=>&$dashboard){
			foreach($dashboard["indicators"] as &$indicator){
				if($id = $this->indicatorService->alreadyExistsOnDb($indicator["indicator"], true)){
					$indicatorData = $this->indicatorService->getIndicatorsViewsAndDatasources(array($id), true)[0];
					$indicator["dataSource"] = $indicatorData["dataSources"][0]["_id"];
					$indicator["indicator"]  = $indicatorData["indicator"]["_id"];
					foreach($indicatorData["views"] as $key=>$widget){
						array_push($viewIds,array("_id"=>(string)$widget["_id"]));
						if($indicator["widget"] === $widget["oldId"]){
							$indicator["widget"] = $widget["_id"];
						}
					}
					$dashboard["dataSource"] = $indicatorData["dataSources"][0]["_id"];
				}else{
					$isImportDefaultOrg = Utilities::getOrganizationIdForMongo() == "5";
					if($isImportDefaultOrg){
						$indicator["mockPeriod"] = !empty($indicator["period"]) ? $this->widgetService->setPeriodFormat($indicator["period"]) : null;
					}
					$imported = $this->indicatorService->saveDefaultOrgIndicatorsAndDataSources(array($indicator["indicator"]));
					if(!empty($imported)){
                        $indicator["dataSource"] = $imported["datasource"][0][0]["_id"];
                        $indicator["indicator"] = $imported["indicator"][0]["_id"];
                        $indicatorImported[] = $imported["indicator"][0];

                        if(!empty($imported["newDatasource"])) {
                            $dataSourceImported[] = $imported["newDatasource"][0];
						}
						if(!empty($imported["widget"][1])){
							foreach($imported["widget"][1] as $key=>$widget){
								array_push($viewIds,array("_id"=>(string)$widget));
								if($indicator["widget"] === $widget){
									$indicator["widget"] = $imported["widget"][0][$key]["_id"];
								}
							}
						}
                        $dashboard["dataSource"] = $imported["datasource"][0][0]["_id"];
                    }
				}
			}
		}
		return array($dashboardsToImport,$viewIds,$indicatorImported,$dataSourceImported);
	}

	protected function setIndicatorFilterParameters($indicator, $filterParams){
		$parameters = array('structureId', 'period', 'dimension', 'facts', 'orderBy', 'limitrows', 'drillDownFilters', 'order', 'color', 'previousConditions', 'postConditions');

		foreach ($parameters as $parameter) {
			$indicator[$parameter] = $this->setIndicatorFilterParameter($indicator, $filterParams, $parameter);
		}

		return $indicator;
	}

	protected function removeObjectsAlreadyImported($objects){
		$alreadyOnDb = $this->organizationManager->findBy(new FilterCriteria($this->dataSourceName,array()))->getRows();
		foreach($objects as $key=>$object){
			foreach($alreadyOnDb as $dbObject){
				if(isset($dbObject['oldId']) && $object['_id'] === $dbObject['oldId']) unset($objects[$key]);
			}
		}
		return $objects;
	}

	protected function setIndicatorFilterParameter($indicator, $filterParams, $parameter){
		if(isset($filterParams[$parameter])){
			return $filterParams[$parameter];
		} else if (isset($indicator[$parameter])){
			return $indicator[$parameter];
		}
	}

	protected function buildDashboardIndicator($dashboardIndicator, $userStructureHierarchy = null, $isViewPreview = false, $hierarchyList = null) {
		$viewId = $dashboardIndicator['widget'];
		$structureId = isset($dashboardIndicator['structureId']) ? $dashboardIndicator['structureId'] : null;
		$period = $dashboardIndicator['period'] ?: array();
		$mockPeriod = ($isViewPreview && !empty($dashboardIndicator['mockPeriod'])) ? $dashboardIndicator['mockPeriod'] : array();
		$dimension = isset($dashboardIndicator['dimension']) ? $dashboardIndicator['dimension'] : null;
		$facts = isset($dashboardIndicator['facts']) ? $dashboardIndicator['facts'] : null;
		$limitRows = !empty($dashboardIndicator['limitrows']) ? $dashboardIndicator['limitrows'] : null;
		$drillDownFilters = isset($dashboardIndicator['drillDownFilters']) ? $dashboardIndicator['drillDownFilters'] : null;
		$order = isset($dashboardIndicator['order']) ? $dashboardIndicator['order'] : null;
		$color = isset($dashboardIndicator['color']) ? $dashboardIndicator['color'] : 7;
		$previousConditions = isset($dashboardIndicator['previousConditions']) ? $dashboardIndicator['previousConditions'] : null;
		$postConditions = isset($dashboardIndicator['postConditions']) ? $dashboardIndicator['postConditions'] : null;
		$displayMode = isset($dashboardIndicator["displayMode"]) ? $dashboardIndicator["displayMode"] : null;
		$dashboardOnClick = isset($dashboardIndicator["actionOnClick"]) ? $dashboardIndicator["actionOnClick"] : null;

		$orderBy = $this->getOrderBy($dashboardIndicator);
		$widget = $this->widgetService->buildWidgetToRender($viewId, $structureId, $period, $dimension, $facts, $orderBy,
															$limitRows, $drillDownFilters, $order, $color, $dashboardOnClick,
															null,$userStructureHierarchy, null, null, null, $isViewPreview, $hierarchyList, $mockPeriod);

		$widget['indicatorDimensions'] = $this->getIndicatorDimensions($dashboardIndicator['indicator']);
		$widget['parameters']['previousConditions'] = $previousConditions ?: null;
		$widget['parameters']['postConditions'] = $postConditions ?: null;
		$widget['displayMode'] = $displayMode ?: null;

		$this->addDashboardIndicatorPropertiers($widget, $dashboardIndicator);

		return $widget;
	}

	protected function getIndicatorDimensions($indicatorId){
		$indicatorDimensions = array();
		$indicator = $this->getIndicatorFromId($indicatorId)[0];
		foreach ($indicator['dataSourceColumns'] as $column) {
			if(isset($column['isDimension']) && $column['isDimension']){
				array_push($indicatorDimensions, $column['name']);
			}
		}
		return $indicatorDimensions;
	}

	protected function getIndicatorFromId($indicatorId){
		$filterCriteria = new FilterCriteria('indicator');
		$filterCriteria->addCondition("_id", $indicatorId);
		$indicator = $this->viewService->findBy($filterCriteria);
		return $indicator;
	}

	protected function setWidgetPropertyIfExists(&$widget, $value, $mainProperty, $secondProperty = null){
		if($value){

			if($secondProperty){
				$widget[$mainProperty][$secondProperty] = $value;
			} else {
				$widget[$mainProperty] = $value;
			}
		}
	}

	protected function addDashboardIndicatorPropertiers(&$widget, $dashboardIndicator) {
		if (!empty($dashboardIndicator['name'])) {
			$widget['label'] = $dashboardIndicator['name'];
		}

		$width = $dashboardIndicator["width"];
		$widget['width'] = $width;

		$height = $dashboardIndicator["height"];
		$widget['height'] = $height;
        if(!empty($widget['FIELD_SIZE']) && !empty($widget['template']) && $widget['template'] = 'widget/grid.html'
                && !empty($widget['fields']) && is_array($widget['fields'])
        ){
            $sizes = explode(',',$widget['FIELD_SIZE']);
            foreach ($sizes as $index => $size) {
                if($size && !empty($widget['fields'][$index])){
                    if(!empty($widget['fields'][$index]['align'])){
                        $widget['fields'][$index]['align'] .= ' zmPer'.trim($size);
                    }else{
                        $widget['fields'][$index]['align'] = 'left zmPer'.trim($size);
                    }
                }
            }
        }
		if (!empty($widget['viewTypeId'])) {
			$viewType = $this->viewTypeService->findById($widget['viewTypeId']);
			$widget['template'] = isset($viewType['dashboardTemplate']) ? $viewType['dashboardTemplate'] : $viewType['template'];
            if(isset($widget['isCrossTab'])){
				$widget['template'] = 'zmart-components#dashboard-grid.html';
            }
		}
	}

	protected function getOrderBy($dashboardIndicator){
		$orderBy = array();
		if (!empty($dashboardIndicator["orderBy"])) {
			$direction = !empty($dashboardIndicator["orderByDirection"]) ? $dashboardIndicator["orderByDirection"] : "ASC";
			$orderBy[$dashboardIndicator["orderBy"]] = $direction;
		}
		return $orderBy;
	}

	public function getDataFromMultipleOrgs(FilterCriteria $filterCriteria){
		$data = array();
		foreach ($this->defaultOrgManagers as $dbLabel=>$defaultOrgManager) {
			$rows = $defaultOrgManager->findBy($filterCriteria)->getRows();
			foreach ($rows as &$row) {
				$row["organizationName"] = $dbLabel;
				array_push($data, $row);
			}

		}
		return $data;
	}
}