<?php
namespace Zeedhi\Notification;

class APITest extends \PHPUnit_Framework_TestCase {

    const ZEEDHI_API_KEY = '123';
    const APPLICATION_ID = '456';

    protected $api;
    protected $stub;

    public function setUp() {
        $this->stub = $this->getMockBuilder('Zeedhi\Notification\cURLRequest')
                           ->setMethods(array('request'))
                           ->setConstructorArgs(array(self::ZEEDHI_API_KEY))
                           ->getMock();
        $this->api = new API(self::ZEEDHI_API_KEY, self::APPLICATION_ID, $this->stub);
    }

    public function testSendMessage() {
        $topicName = 'topicName';
        $notification = array(
            'title'   => 'Notification Test',
            'message' => 'test'
        );

        $this->stub->expects($this->once())
            ->method('request')
            ->with('/message/send/', array(
                    'applicationId' => self::APPLICATION_ID,
                    'topicName'     => $topicName,
                    'notification'  => json_encode($notification)
                ))
            ->willReturn('{
                    "success": true,
                    "dataset": {
                        "message": [
                            {
                                "_id"          : "123",
                                "topicId"      : "456",
                                "notification" : {"title": "Title", "message": "Message"},
                                "nrReceived"   : 0,
                                "nrRead"       : 10
                            }
                        ]
                    }
                }');

        $expected = new Message(
            '123',
            '456',
            array('title' => 'Title', 'message' => 'Message'),
            0,
            10
        );
        $this->assertEquals($expected, $this->api->sendMessage($topicName, $notification));
    }

    public function testGetAllTopics() {
        $topicName = 'topicName';

        $this->stub->expects($this->once())
             ->method('request')
             ->with('/topic/', array('applicationId' => self::APPLICATION_ID))
             ->willReturn('{
                    "success": true,
                    "dataset": {
                        "topic": [
                            {
                                "_id": 123,
                                "name": "'.$topicName.'",
                                "nrSubscribers": 0
                            }
                        ]
                    }
                }');

        $expected = array(
            new Topic('123', $topicName, 0)
        );
        $this->assertEquals($expected, $this->api->getAllTopics());
    }

    public function testGetAllMessages() {
        $topicName = 'topicName';

        $this->stub->expects($this->once())
             ->method('request')
             ->with('/message/', array(
                        'applicationId' => self::APPLICATION_ID,
                        'topicName'     => $topicName
                    ))
             ->willReturn('{
                    "success": true,
                    "dataset": {
                        "message": [
                            {
                                "_id"          : "123",
                                "topicId"      : "456",
                                "notification" : {"_id": "123", "title": "Notification Test", "message": "test"},
                                "nrReceived"   : 12,
                                "nrRead"       : 32
                            }
                        ]
                    }
                }');

        $expected = array(
            new Message(
                '123',
                '456',
                array('_id' => '123', 'title' => 'Notification Test', 'message' => 'test'),
                12,
                32
            )
        );
        $this->assertEquals($expected, $this->api->getAllMessages($topicName));
    }

    public function testCreateTopic() {
        $topicName = 'topicName';

        $this->stub->expects($this->once())
             ->method('request')
             ->with('/topic/create/', array(
                        'applicationId' => self::APPLICATION_ID,
                        'topicName'     => $topicName
                    ))
             ->willReturn('{
                    "success": true,
                    "dataset": {
                        "topic": [
                            {
                                "_id"           : "123",
                                "name"          : "'.$topicName.'",
                                "nrSubscribers" : 54
                            }
                        ]
                    }
                }');

        $expected = new Topic('123', $topicName, 54);
        $this->assertEquals($expected, $this->api->createTopic($topicName));
    }

    /**
     *
     * @expectedException Zeedhi\Notification\Exception
     * @expectedExceptionMessage A error happen in request: Invalid topic
     *
     */
    public function testErrorRequest() {
        $topicName = 'invalidTopicName';
        $this->stub->expects($this->once())
             ->method('request')
             ->with('/message/')
             ->willReturn('{
                    "success": false,
                    "error": "Invalid topic"
                }');

        $this->api->getAllMessages($topicName);
    }

    public function testFactoryAPI() {
        $this->assertInstanceOf(
            'Zeedhi\Notification\API',
            API::factoryAPI(self::ZEEDHI_API_KEY, self::APPLICATION_ID, $this->stub));
    }
}